from baseapi import APIResource


class ipam:
    def __init__(self, client):
        self.aggregates = self._aggregates(client)
        self.asns = self._asns(client)
        self.fhrp_group_assignments = self._fhrp_group_assignments(client)
        self.fhrp_groups = self._fhrp_groups(client)
        self.ip_addresss = self._ip_addresss(client)
        self.ip_ranges = self._ip_ranges(client)
        self.l2vpn_terminations = self._l2vpn_terminations(client)
        self.l2vpns = self._l2vpns(client)
        self.prefixs = self._prefixs(client)
        self.rirs = self._rirs(client)
        self.roles = self._roles(client)
        self.route_targets = self._route_targets(client)
        self.service_templates = self._service_templates(client)
        self.services = self._services(client)
        self.vlan_groups = self._vlan_groups(client)
        self.vlans = self._vlans(client)
        self.vrfs = self._vrfs(client)
        super().__init__()

    class _aggregates(APIResource):
        path = "ipam/aggregates/"

    class _asns(APIResource):
        path = "ipam/asns/"

    class _fhrp_group_assignments(APIResource):
        path = "ipam/fhrp-group-assignments/"

    class _fhrp_groups(APIResource):
        path = "ipam/fhrp-groups/"

    class _ip_addresss(APIResource):
        path = "ipam/ip-addresses/"

    class _ip_ranges(APIResource):
        path = "ipam/ip-ranges/"

    class _l2vpn_terminations(APIResource):
        path = "ipam/l2vpn-terminations/"

    class _l2vpns(APIResource):
        path = "ipam/l2vpns/"

    class _prefixs(APIResource):
        path = "ipam/prefixes/"

    class _rirs(APIResource):
        path = "ipam/rirs/"

    class _roles(APIResource):
        path = "ipam/roles/"

    class _route_targets(APIResource):
        path = "ipam/route-targets/"

    class _service_templates(APIResource):
        path = "ipam/service-templates/"

    class _services(APIResource):
        path = "ipam/services/"

    class _vlan_groups(APIResource):
        path = "ipam/vlan-groups/"

    class _vlans(APIResource):
        path = "ipam/vlans/"

    class _vrfs(APIResource):
        path = "ipam/vrfs/"



from baseapi import APIResource


class dcim:
    def __init__(self, client):
        self.cable_terminations = self._cable_terminations(client)
        self.cables = self._cables(client)
        self.connected_devices = self._connected_devices(client)
        self.console_port_templates = self._console_port_templates(client)
        self.console_ports = self._console_ports(client)
        self.console_server_port_templates = self._console_server_port_templates(client)
        self.console_server_ports = self._console_server_ports(client)
        self.device_bay_templates = self._device_bay_templates(client)
        self.device_bays = self._device_bays(client)
        self.device_roles = self._device_roles(client)
        self.device_types = self._device_types(client)
        self.devices = self._devices(client)
        self.front_port_templates = self._front_port_templates(client)
        self.front_ports = self._front_ports(client)
        self.interface_templates = self._interface_templates(client)
        self.interfaces = self._interfaces(client)
        self.inventory_item_roles = self._inventory_item_roles(client)
        self.inventory_item_templates = self._inventory_item_templates(client)
        self.inventory_items = self._inventory_items(client)
        self.locations = self._locations(client)
        self.manufacturers = self._manufacturers(client)
        self.module_bay_templates = self._module_bay_templates(client)
        self.module_bays = self._module_bays(client)
        self.module_types = self._module_types(client)
        self.modules = self._modules(client)
        self.platforms = self._platforms(client)
        self.power_feeds = self._power_feeds(client)
        self.power_outlet_templates = self._power_outlet_templates(client)
        self.power_outlets = self._power_outlets(client)
        self.power_panels = self._power_panels(client)
        self.power_port_templates = self._power_port_templates(client)
        self.power_ports = self._power_ports(client)
        self.rack_reservations = self._rack_reservations(client)
        self.rack_roles = self._rack_roles(client)
        self.racks = self._racks(client)
        self.rear_port_templates = self._rear_port_templates(client)
        self.rear_ports = self._rear_ports(client)
        self.regions = self._regions(client)
        self.site_groups = self._site_groups(client)
        self.sites = self._sites(client)
        self.virtual_chassiss = self._virtual_chassiss(client)
        self.virtual_device_contexts = self._virtual_device_contexts(client)
        super().__init__()

    class _cable_terminations(APIResource):
        path = "dcim/cable-terminations/"

    class _cables(APIResource):
        path = "dcim/cables/"

    class _connected_devices(APIResource):
        path = "dcim/connected-device/"

    class _console_port_templates(APIResource):
        path = "dcim/console-port-templates/"

    class _console_ports(APIResource):
        path = "dcim/console-ports/"

    class _console_server_port_templates(APIResource):
        path = "dcim/console-server-port-templates/"

    class _console_server_ports(APIResource):
        path = "dcim/console-server-ports/"

    class _device_bay_templates(APIResource):
        path = "dcim/device-bay-templates/"

    class _device_bays(APIResource):
        path = "dcim/device-bays/"

    class _device_roles(APIResource):
        path = "dcim/device-roles/"

    class _device_types(APIResource):
        path = "dcim/device-types/"

    class _devices(APIResource):
        path = "dcim/devices/"

    class _front_port_templates(APIResource):
        path = "dcim/front-port-templates/"

    class _front_ports(APIResource):
        path = "dcim/front-ports/"

    class _interface_templates(APIResource):
        path = "dcim/interface-templates/"

    class _interfaces(APIResource):
        path = "dcim/interfaces/"

    class _inventory_item_roles(APIResource):
        path = "dcim/inventory-item-roles/"

    class _inventory_item_templates(APIResource):
        path = "dcim/inventory-item-templates/"

    class _inventory_items(APIResource):
        path = "dcim/inventory-items/"

    class _locations(APIResource):
        path = "dcim/locations/"

    class _manufacturers(APIResource):
        path = "dcim/manufacturers/"

    class _module_bay_templates(APIResource):
        path = "dcim/module-bay-templates/"

    class _module_bays(APIResource):
        path = "dcim/module-bays/"

    class _module_types(APIResource):
        path = "dcim/module-types/"

    class _modules(APIResource):
        path = "dcim/modules/"

    class _platforms(APIResource):
        path = "dcim/platforms/"

    class _power_feeds(APIResource):
        path = "dcim/power-feeds/"

    class _power_outlet_templates(APIResource):
        path = "dcim/power-outlet-templates/"

    class _power_outlets(APIResource):
        path = "dcim/power-outlets/"

    class _power_panels(APIResource):
        path = "dcim/power-panels/"

    class _power_port_templates(APIResource):
        path = "dcim/power-port-templates/"

    class _power_ports(APIResource):
        path = "dcim/power-ports/"

    class _rack_reservations(APIResource):
        path = "dcim/rack-reservations/"

    class _rack_roles(APIResource):
        path = "dcim/rack-roles/"

    class _racks(APIResource):
        path = "dcim/racks/"

    class _rear_port_templates(APIResource):
        path = "dcim/rear-port-templates/"

    class _rear_ports(APIResource):
        path = "dcim/rear-ports/"

    class _regions(APIResource):
        path = "dcim/regions/"

    class _site_groups(APIResource):
        path = "dcim/site-groups/"

    class _sites(APIResource):
        path = "dcim/sites/"

    class _virtual_chassiss(APIResource):
        path = "dcim/virtual-chassis/"

    class _virtual_device_contexts(APIResource):
        path = "dcim/virtual-device-contexts/"



from baseapi import APIResource


class users:
    def __init__(self, client):
        self.configs = self._configs(client)
        self.groups = self._groups(client)
        self.permissions = self._permissions(client)
        self.tokens = self._tokens(client)
        self.users = self._users(client)
        super().__init__()

    class _configs(APIResource):
        path = "users/config/"

    class _groups(APIResource):
        path = "users/groups/"

    class _permissions(APIResource):
        path = "users/permissions/"

    class _tokens(APIResource):
        path = "users/tokens/"

    class _users(APIResource):
        path = "users/users/"



from typing import Any, Dict, List, Union

from api.circuits import circuits
from api.core import core
from api.dcim import dcim
from api.extras import extras
from api.ipam import ipam
from api.plugins import plugins
from api.tenancy import tenancy
from api.users import users
from api.virtualization import virtualization
from api.wireless import wireless
from rest import RestClient
from baseapi import RetrievableRootAPIResource, baseapi

NETBOX_DEFAULT_HEADERS = {
    "Accept": "application/json;",
    "User-Agent": "python-requests",
}

JSONType = Union[None, bool, int, float, str, List[Any], Dict[str, Any]]


class NetBoxClient(RestClient):
    def __init__(self, base_url: str, token: str, headers: Dict[str, str] = None):
        self.status = self._status(self)
        self.token = token

        headers = headers or NETBOX_DEFAULT_HEADERS
        if token:
            headers["authorization"] = f"Token {token}"

        url = base_url = "{}/api".format(
            base_url if base_url[-1] != "/" else base_url[:-1]
        )

        self.circuits = circuits(self)
        self.core = core(self)
        self.dcim = dcim(self)
        self.extras = extras(self)
        self.ipam = ipam(self)
        self.plugins = plugins(self)
        self.tenancy = tenancy(self)
        self.users = users(self)
        self.virtualization = virtualization(self)
        self.wireless = wireless(self)

        super().__init__(base_url=url, headers=headers)

    def __enter__(self):
        return self

    class _status(baseapi, RetrievableRootAPIResource):
        path = "status/"



from netbox import NetBoxClient
import json

nb = NetBoxClient(
    base_url="http://127.0.0.1:8000/", token="1dc6fa5bfcef8390dd83a261c36ed8f1551b2d6b"
)


def print_response(response):
    print(f"status: {response.status_code}")
    print(response.data)


# response = nb.schema.get(format="yaml")
# print_response(response)
# response = nb.status.get()
# print_response(response)

print("")
print("1. List")
print("--------------------------------------------")
ret = nb.dcim.sites.list(limit=3)
print(json.dumps(ret.data, indent=4))

"""
print("")
print("2. Filtered List")
print("--------------------------------------------")
sites = nb.dcim.sites.list(region_id="43")
print(sites)

print("")
print("3. Get")
print("--------------------------------------------")
site = nb.dcim.sites.get(24)
print(site)

print("")
print("4. Create")
print("--------------------------------------------")
ret = nb.dcim.sites.create(name="foo3", slug="foo3")
print(ret)

print("")
print("5. Bulk Create")
print("--------------------------------------------")
data = [
    {"name": "foo4", "slug": "foo4"},
    {"name": "foo5", "slug": "foo5"},
    {"name": "foo6", "slug": "foo6"},
]
ret = nb.dcim.sites.create(data)
print(ret)

print("")
print("6. Update")
print("--------------------------------------------")
ret = nb.dcim.sites.update(26, name="foo2-new", slug="foo2-new-slug")
print(ret)

print("")
print("7. Bulk Update")
print("--------------------------------------------")
data = [
    {"id": 28, "name": "foo4-new", "slug": "foo4-new"},
    {"id": 29, "name": "foo5-new", "slug": "foo5-new"},
]
ret = nb.dcim.sites.update(data)
print(ret)

print("")
print("8. Delete")
print("--------------------------------------------")
response = nb.dcim.sites.delete(37)
print_response(response)

print("")
print("9. Bulk Delete")
print("--------------------------------------------")
data = [{"id": 25}, {"id": 27}]
response = nb.dcim.sites.delete(data)
print_response(response)
"""


# print(nb.circuits.circuits.list())
# circuit = nb.circuits.circuits.get(19)
# print("")
# print(type(circuit))
# print(circuit)
# print(circuit["id"])
# import json
#
# print(json.dumps(circuit, indent=4))
# sites = nb.dcim.sites.list()
# print(json.dumps(sites, indent=4))
# nb.dcim.sites.create(name="foo2", slug="foo2")

# racks = nb.dcim.racks.list()
# print(json.dumps(racks, indent=4))



from functools import partialmethod
from json import JSONDecodeError
from typing import Any, Dict, List, Union

import requests
from exceptions import NetBoxException
from requests.structures import CaseInsensitiveDict

JSONType = Union[None, bool, int, float, str, List[Any], Dict[str, Any]]


class Result:
    def __init__(
        self,
        status_code: int,
        headers: CaseInsensitiveDict,
        message: str = "",
        pagination: dict = None,
        data: List[Dict] = None,
    ):
        """
        Result returned from low-level RestAdapter
        :param status_code: Standard HTTP Status code
        :param message: Human readable result
        :param data: Python List of Dictionaries (or maybe just a single Dictionary on error)
        """
        self.status_code = int(status_code)
        self.headers = headers
        self.message = str(message)
        self.data = data if data else []


class RestClient:
    def __init__(self, base_url: str, **session_kwargs):
        self.base_url = base_url
        self._session = requests.Session()

        for key, value in session_kwargs.items():
            setattr(self._session, key, value)

    def __enter__(self):
        return self

    def __exit__(self, *_):
        self.close()

    def close(self):
        return self._session.close()

    def request(self, method: str, path: str, **kwargs) -> JSONType:
        path = f"{self.base_url}/{path}"
        data_out = None
        try:
            response = self._session.request(method=method, url=path, **kwargs)
            response.raise_for_status()
        except requests.HTTPError as http_error:
            raise NetBoxException(
                f"Invalid request from {self.base_url}: {http_error}"
            ) from http_error
        except requests.RequestException as err:
            # self._logger.error(msg=(str(err)))
            raise NetBoxException("Request failed") from err

        if method != "DELETE":
            # Deserialize JSON output to Python object, or return failed Result on exception
            try:
                data_out = response.json()
            except (ValueError, TypeError, JSONDecodeError) as err:
                # self._logger.error(msg=log_line_post.format(False, None, e))
                raise NetBoxException("Bad JSON in response") from err

        # If status_code in 200-299 range, return success Result with data, otherwise raise exception
        is_success = 299 >= response.status_code >= 200  # 200 to 299 is OK
        # log_line = log_line_post.format(is_success, response.status_code, response.reason)
        if is_success:
            # self._logger.debug(msg=log_line)
            # check if list - fixme: should have cleaner way to do this
            pagination = None
            if "count" in data_out and "results" in data_out:
                pagination = {
                    "count": data_out.get("count"),
                    "next": data_out.get("next"),
                    "previous": data_out.get("previous")
                }
                data_out = data_out.get("results")

            return Result(
                response.status_code,
                headers=response.headers,
                message=response.reason,
                pagination = pagination,
                data=data_out,
            )
        # self._logger.error(msg=log_line)
        raise NetBoxException(f"{response.status_code}: {response.reason}")

    get = partialmethod(request, "GET")
    post = partialmethod(request, "POST")
    put = partialmethod(request, "PUT")
    patch = partialmethod(request, "PATCH")
    delete = partialmethod(request, "DELETE")
    head = partialmethod(request, "HEAD")
    options = partialmethod(request, "OPTIONS")



from baseapi import APIResource


class wireless:
    def __init__(self, client):
        self.wireless_lan_groups = self._wireless_lan_groups(client)
        self.wireless_lans = self._wireless_lans(client)
        self.wireless_links = self._wireless_links(client)
        super().__init__()

    class _wireless_lan_groups(APIResource):
        path = "wireless/wireless-lan-groups/"

    class _wireless_lans(APIResource):
        path = "wireless/wireless-lans/"

    class _wireless_links(APIResource):
        path = "wireless/wireless-links/"



from baseapi import APIResource


class circuits:
    def __init__(self, client):
        self.circuit_terminations = self._circuit_terminations(client)
        self.circuit_types = self._circuit_types(client)
        self.circuits = self._circuits(client)
        self.provider_networks = self._provider_networks(client)
        self.providers = self._providers(client)

        super().__init__()

    class _circuit_terminations(APIResource):
        path = "circuits/circuit-terminations/"

    class _circuit_types(APIResource):
        path = "circuits/circuit-types/"

    class _circuits(APIResource):
        path = "circuits/circuits/"

    class _provider_networks(APIResource):
        path = "circuits/provider_networks/"

    class _providers(APIResource):
        path = "circuits/providers/"



class baseapi:
    def __init__(self, client):
        self.client = client
        super().__init__()


class CreateableAPIResource:
    def create(self, *args, **kwargs):
        return self.client.post(self.path, json=args[0] if args else kwargs)


class DeletableAPIResource:
    def delete(self, obj):
        if obj and isinstance(obj, list):
            return self.client.delete(self.path, json=obj)
        return self.client.delete(f"{self.path}{obj}/")


class ListableAPIResource:
    def list(self, **kwargs):
        return self.client.get(self.path, params=kwargs)


class RetrievableAPIResource:
    def get(self, id):
        return self.client.get(f"{self.path}{id}/")


class RetrievableRootAPIResource:
    def get(self, **kwargs):
        return self.client.get(self.path, params=kwargs)


class UpdateableAPIResource:
    def update(self, obj, **kwargs):
        if obj and isinstance(obj, list):
            return self.client.patch(self.path, json=obj)
        return self.client.patch(f"{self.path}{obj}/", json=kwargs)


class APIResource(
    baseapi,
    CreateableAPIResource,
    DeletableAPIResource,
    ListableAPIResource,
    RetrievableAPIResource,
    UpdateableAPIResource,
):
    pass


class ROAPIResource(
    baseapi,
    DeletableAPIResource,
    ListableAPIResource,
    RetrievableAPIResource,
    UpdateableAPIResource,
):
    pass



from netbox import NetBoxClient
import pandas as pd
from tabulate import tabulate
from typing import List
from enum import Enum
import json
import yaml


class ListFormat(str, Enum):
    text = "text"
    json = "json"
    yaml = "yaml"
    csv = "csv"


class TableFormat(str, Enum):
    plain = "plain"
    simple = "simple"
    github = "github"
    grid = "grid"
    simple_grid = "simple_grid"
    rounded_grid = "rounded_grid"
    heavy_grid = "heavy_grid"
    mixed_grid = "mixed_grid"
    double_grid = "double_grid"
    fancy_grid = "fancy_grid"
    outline = "outline"
    simple_outline = "simple_outline"
    rounded_outline = "rounded_outline"
    heavy_outline = "heavy_outline"
    mixed_outline = "mixed_outline"
    double_outline = "double_outline"
    fancy_outline = "fancy_outline"
    pipe = "pipe"
    orgtbl = "orgtbl"
    asciidoc = "asciidoc"
    jira = "jira"
    presto = "presto"
    pretty = "pretty"
    psql = "psql"
    rst = "rst"
    mediawiki = "mediawiki"
    moinmoin = "moinmoin"
    youtrack = "youtrack"
    html = "html"
    unsafehtml = "unsafehtml"
    latex = "latex"
    latex_raw = "latex_raw"
    latex_booktabs = "latex_booktabs"
    latex_longtable = "latex_longtable"
    textile = "textile"
    tsv = "tsv"


class BaseCrudCommand:
    connection = NetBoxClient(
        base_url="http://127.0.0.1:8000/",
        token="1dc6fa5bfcef8390dd83a261c36ed8f1551b2d6b",
    )

    def list(self, format: ListFormat, columns: str, indent: int, tablefmt: TableFormat):
        data = self.base_api.list().data["results"]
        if format == ListFormat.text:
            data = pd.json_normalize(data, sep="__")
            columns = columns.split(",")
            data = data[data.columns.intersection(columns)]
            print(tabulate(data, headers="keys", tablefmt=tablefmt, showindex=False))
            return

        if format == ListFormat.json:
            print(json.dumps(data, indent=indent))
            return

        if format == ListFormat.yaml:
            print(yaml.dump(data))
            return

        if format == ListFormat.csv:
            data = pd.json_normalize(data, sep="__")
            columns = columns.split(",")
            print(data.to_csv(index=False, columns=columns))
            return

    def get(self, func):
        pass

    def create(self, func):
        pass

    def update(self, func):
        pass

    def delete(self, func):
        pass



from baseapi import APIResource


class virtualization:
    def __init__(self, client):
        self.cluster_groups = self._cluster_groups(client)
        self.cluster_types = self._cluster_types(client)
        self.clusters = self._clusters(client)
        self.interfaces = self._interfaces(client)
        self.virtual_machines = self._virtual_machines(client)
        super().__init__()

    class _cluster_groups(APIResource):
        path = "virtualization/cluster-groups/"

    class _cluster_types(APIResource):
        path = "virtualization/cluster-types/"

    class _clusters(APIResource):
        path = "virtualization/clusters/"

    class _interfaces(APIResource):
        path = "virtualization/interfaces/"

    class _virtual_machines(APIResource):
        path = "virtualization/virtual-machines/"



from api.circuits import circuits
from api.core import core
from api.dcim import dcim
from api.extras import extras
from api.ipam import ipam
from api.plugins import plugins
from api.tenancy import tenancy
from api.users import users
from api.virtualization import virtualization
from api.wireless import wireless



from baseapi import APIResource


class plugins:
    def __init__(self, client):
        self.installed_plugins = self._installed_plugins(client)
        super().__init__()

    class _installed_plugins(APIResource):
        path = "plugins/installed-plugins/"



import typer

from . import circuit_terminations

# self.circuit_terminations = self._circuit_terminations(client)
# self.circuit_types = self._circuit_types(client)
# self.circuits = self._circuits(client)
# self.provider_networks = self._provider_networks(client)
# self.providers = self._providers(client)


app = typer.Typer()
app.add_typer(circuit_terminations.app, name="circuit_terminations")



#!/usr/bin/env python

"""The setup script."""

from setuptools import find_packages, setup

with open("README.md") as readme_file:
    readme = readme_file.read()

requirements = []

test_requirements = [
    "pytest>=3",
]

setup(
    author="Arthur Hanson",
    author_email="ahanson@netboxlabs.com",
    python_requires=">=3.8",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    description="NetBox Python API Client",
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=readme,
    long_description_content_type='text/markdown',
    include_package_data=True,
    keywords="netbox_python",
    name="netbox_python",
    packages=find_packages(include=["netbox_python", "netbox_python.*"]),
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/netbox-community/netbox_python",
    version="0.1.0",
    zip_safe=False,
)



from baseapi import APIResource


class core:
    def __init__(self, client):
        self.data_files = self._data_files(client)
        self.data_sourcess = self._data_sources(client)
        super().__init__()

    class _data_files(APIResource):
        path = "core/data-files/"

    class _data_sources(APIResource):
        path = "core/data-sources/"



import typer

from netbox import NetBoxClient
import pandas as pd
from tabulate import tabulate
from cli.crud import BaseCrudCommand, ListFormat, TableFormat
from typing import List


class CircuitTermination(BaseCrudCommand):
    @property
    def base_api(self):
        return self.connection.circuits.circuit_terminations


app = typer.Typer()


@app.command()
def list(
    format: ListFormat = typer.Option(ListFormat.json, prompt=True),
    columns: str = typer.Option("id"),
    indent: int = typer.Option(4),
    tablefmt: TableFormat = typer.Option(TableFormat.psql),
):
    CircuitTermination().list(format, columns, indent, tablefmt)


def get():
    CircuitTermination().get()


def create():
    CircuitTermination().create()


def update():
    CircuitTermination().update()


def delete():
    CircuitTermination().delete()



from baseapi import APIResource


class tenancy:
    def __init__(self, client):
        self.contact_assignments = self._contact_assignments(client)
        self.contact_groups = self._contact_groups(client)
        self.contact_roles = self._contact_roles(client)
        self.contacts = self._contacts(client)
        self.tenant_groups = self._tenant_groups(client)
        self.tenants = self._tenants(client)
        super().__init__()

    class _contact_assignments(APIResource):
        path = "tenancy/contact-assignments/"

    class _contact_groups(APIResource):
        path = "tenancy/contact-groups/"

    class _contact_roles(APIResource):
        path = "tenancy/contact-roles/"

    class _contacts(APIResource):
        path = "tenancy/contacts/"

    class _tenant_groups(APIResource):
        path = "tenancy/tenant-groups/"

    class _tenants(APIResource):
        path = "tenancy/tenants/"



from baseapi import APIResource


class extras:
    def __init__(self, client):
        self.config_contexts = self._config_contexts(client)
        self.config_templates = self._config_templates(client)
        self.content_types = self._content_types(client)
        self.custom_fields = self._custom_fields(client)
        self.custom_links = self._custom_links(client)
        self.export_templates = self._export_templates(client)
        self.image_attachments = self._image_attachments(client)
        self.job_results = self._job_results(client)
        self.journal_entries = self._journal_entries(client)
        self.object_changes = self._object_changes(client)
        self.reports = self._reports(client)
        self.saved_filters = self._saved_filters(client)
        self.scripts = self._scripts(client)
        self.tags = self._tags(client)
        self.webhooks = self._webhooks(client)
        super().__init__()

    class _config_contexts(APIResource):
        path = "extras/config-contexts/"

    class _config_templates(APIResource):
        path = "extras/config-templates/"

    class _content_types(APIResource):
        path = "extras/content-types/"

    class _custom_fields(APIResource):
        path = "extras/custom-fields/"

    class _custom_links(APIResource):
        path = "extras/custom-links/"

    class _export_templates(APIResource):
        path = "extras/export-templates/"

    class _image_attachments(APIResource):
        path = "extras/image-attachments/"

    class _job_results(APIResource):
        path = "extras/job-results/"

    class _journal_entries(APIResource):
        path = "extras/journal-entries/"

    class _object_changes(APIResource):
        path = "extras/object-changes/"

    class _reports(APIResource):
        path = "extras/reports/"

    class _saved_filters(APIResource):
        path = "extras/saved-filters/"

    class _scripts(APIResource):
        path = "extras/scripts/"

    class _tags(APIResource):
        path = "extras/tags/"

    class _webhooks(APIResource):
        path = "extras/webhooks/"
